@@ -172,6 +172,9 @@ class Operand:
         "zmm"
             A 512-bit ZMM SIMD register (zmm0-zmm31).
 
+        "k"
+            An AVX-512 mask register (k1-k7).
+
         "m"
             A memory operand of any size.
 
@@ -239,7 +242,7 @@ class Operand:
     @property
     def is_register(self):
         """Indicates whether this operand specifies a register"""
-        return self.type in {"al", "cl", "ax", "eax", "rax", "xmm0", "r8", "r16", "r32", "r64", "r8l", "r16l", "r32l", "mm", "xmm", "ymm"}
+        return self.type in {"al", "cl", "ax", "eax", "rax", "xmm0", "r8", "r16", "r32", "r64", "r8l", "r16l", "r32l", "mm", "xmm", "ymm", "zmm", "k"}
 
     @property
     def is_memory(self):
@@ -248,6 +251,45 @@ class Operand:
 
 
 class ISAExtension:
+    _score_map = {
+        "CPUID": 1,
+        "RDTSC": 5,
+        "RDTSCP": 6,
+        "CMOV": 20,
+        "POPCNT": 100,
+        "LZCNT": 101,
+        "TBM": 102,
+        "BMI": 103,
+        "BMI2": 104,
+        "ADX": 105,
+        "MMX": 30,
+        "MMX+": 31,
+        "FEMMS": 40,
+        "3dnow!": 41,
+        "3dnow!+": 42,
+        "SSE": 50,
+        "SSE2": 51,
+        "SSE3": 52,
+        "SSSE3": 53,
+        "SSE4A": 54,
+        "SSE4.1": 55,
+        "SSE4.2": 56,
+        "FMA3": 60,
+        "FMA4": 61,
+        "XOP": 62,
+        "F16C": 63,
+        "AVX": 70,
+        "AVX2": 71,
+        "AVX512F": 72,
+        "AVX512BW": 73,
+        "AVX512DQ": 74,
+        "RDRAND": 80,
+        "RDSEED": 81,
+        "PCLMULQDQ": 90,
+        "AES": 91,
+        "SHA": 92,
+    }
+
     """An extension to x86-64 instruction set.
 
     :ivar name: name of the ISA extension. Possible values are:
@@ -275,6 +317,9 @@ class ISAExtension:
         - "SSE4A"     := Streaming SIMD Extension 4a.
         - "AVX"       := Advanced Vector eXtension.
         - "AVX2"      := Advanced Vector eXtension 2.
+        - "AVX512F"   := AVX-512 Foundation instructions.
+        - "AVX512BW"  := AVX-512 Byte and Word instructions.
+        - "AVX512DQ"  := AVX-512 Doubleword and Quadword instructions.
         - "XOP"       := eXtended OPerations extension.
         - "F16C"      := Half-Precision (F16) Conversion instructions.
         - "FMA3"      := Fused Multiply-Add instructions (3-operand).
@@ -294,12 +339,29 @@ class ISAExtension:
         """Returns the name of the ISA extension"""
         return self.name
 
+    def __repr__(self):
+        return str(self)
+
+    def __eq__(self, other):
+        return isinstance(other, ISAExtension) and self.name == other.name
+
+    def __ne__(self, other):
+        return not isinstance(other, ISAExtension) or self.name != other.name
+
+    def __hash__(self):
+        return hash(self.name)
+
+    @property
+    def score(self):
+        """A number that can be used to order a list of ISA extensions"""
+        return self._score_map.get(self.name, 0)
+
 
 class Encoding:
     """Instruction encoding
 
     :ivar components: a list of :class:`Prefix`, :class:`REX`, :class:`VEX`, :class:`Opcode`, :class:`ModRM`, \
-        :class:`RegisterByte`, :class:`Immediate`, :class:`DataOffset32`, :class:`CodeOffset` objects that \
+        :class:`RegisterByte`, :class:`Immediate`, :class:`DataOffset`, :class:`CodeOffset` objects that \
         specify the components of encoded instruction
     """
 
@@ -485,7 +547,7 @@ class VEX:
         self.B = None
         self.vvvv = None
 
-    def set_ignored(self, w=0, l=0, r=0, x=0, b=0):
+    def set_ignored(self, w=0, l=0, r=1, x=1, b=1):
         """Sets values for ignored bits
 
         :param int w: the value (0 or 1) to be assigned to VEX.W bit if it is ignored.
@@ -583,7 +645,7 @@ class ModRM:
 class Immediate:
     """Immediate constant embedded into instruction encoding.
 
-    :ivar size: size of the constant. Possible values are 1, 2, 4, or 8.
+    :ivar size: size of the constant in bytes. Possible values are 1, 2, 4, or 8.
     :ivar value: value of the constant. Can be an int value or a reference to an instruction operand.
 
         If value is a reference to an instruction operand, the operand has "imm" type of the matching size.
@@ -604,7 +666,7 @@ class CodeOffset:
 
     Offset is relative to the end of the instruction.
 
-    :ivar size: size of the offset. Possible values are 1 or 4.
+    :ivar size: size of the offset in bytes. Possible values are 1 or 4.
     :ivar value: value of the offset. Must be a reference to an instruction operand.
 
         The instruction operand has "rel" type of the matching size.
@@ -619,7 +681,7 @@ class DataOffset:
 
     Only MOV instruction has forms that use direct data offset.
 
-    :ivar size: size of the offset. Possible values are 4 or 8.
+    :ivar size: size of the offset in bytes. Possible values are 4 or 8.
     :ivar value: value of the offset. Must be a reference to an instruction operand.
 
         The instruction operand has "moffs" type of the matching size.
@@ -630,6 +692,10 @@ class DataOffset:
 
 
 def read_instruction_set(filename=os.path.join(os.path.dirname(os.path.abspath(__file__)), "x86_64.xml")):
+    """Reads instruction set data from an XML file and returns a list of :class:`Instruction` objects
+
+    :param filename: path to an XML file with instruction set data
+    """
     xml_tree = ET.parse(filename)
     xml_instruction_set = xml_tree.getroot()
     assert xml_instruction_set.tag == "InstructionSet"
@@ -750,7 +816,7 @@ def read_instruction_set(filename=os.path.join(os.path.dirname(os.path.abspath(_
                             if vex.R == "ignored":
                                 vex.R = None
                             else:
-                                vex.R = int(rex.R)
+                                vex.R = int(vex.R)
                         else:
                             assert "R-operand-number" in xml_component.attrib
                             vex.R = instruction_form.operands[int(xml_component.attrib["R-operand-number"])]
@@ -762,7 +828,7 @@ def read_instruction_set(filename=os.path.join(os.path.dirname(os.path.abspath(_
                             if vex.B == "ignored":
                                 vex.B = None
                             else:
-                                vex.B = int(rex.B)
+                                vex.B = int(vex.B)
                         elif "B-operand-number" in xml_component.attrib:
                             assert "B" not in xml_component.attrib
                             assert "BX-operand-number" not in xml_component.attrib
